if jit and jit.off then jit.off() end
--lua入口
-----------------------------------------------------------------------
function clone(object)
    local lookup_table = {};
    local function _copy(object)
        if(type(object) ~= "table")then
            return object;
        elseif(lookup_table[object])then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for key, value in pairs(object) do
            new_table[_copy(key)] = _copy(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _copy(object);
end

--Create an class.
function class(classname, super)
    local superType = type(super);
    local cls;

    if(superType ~= "function" and superType ~= "table")then
        superType = nil;
        super = nil ;
    end

    cls = {};
    cls.__cname = classname;
    cls.__ctype = 1;

    local superClassName = "";

    function cls.extend(...)
        local instance = super(...);
        if(instance.__cname ~= nil)then
            superClassName = instance.__cname;
        end
        -- copy fields from class to native object
        for k,v in pairs(cls) do
            if(type(v) == "function" and instance[k] ~= nil)then
                instance[superClassName.."_"..k] = instance[k];
                instance[k] = v;
            else
                instance[k] = v ;
            end
        end
        return instance;
    end

    function cls.new(...)
        local instance = cls.extend(...);
        instance.class = cls;
        if(instance.ctor ~= nil)then
            instance:ctor(...);
        end
        return instance;
    end

    return cls;
end

local setmetatableindex_;
setmetatableindex_ = function(t, index) 
    local mt = getmetatable(t);
    if(not mt)then mt = {}; end
    if(not mt.__index)then
        mt.__index = index;
        setmetatable(t, mt);
    elseif(mt.__index ~= index)then
        setmetatableindex_(mt, index);
    end
end
setmetatableindex = setmetatableindex_;

function class2(classname, super)
    local cls = {__cname = classname};
 
    cls.super = super;
    cls.__index = cls;
    if(super)then
        setmetatable(cls, {__index = cls.super});
    end

    cls.new = function(_,...)
        local instance;
        instance = {};
        setmetatableindex(instance, cls);
        instance.class = cls;
        if(instance.ctor ~= nil)then
            instance:ctor(...);
        end
        return instance;
    end

    return cls;
end

-----------------------------------------------------------------------
local json = require "cjson";
local gameSDK;
local gameConfig;
local gameScene;
local isStart = false;

globalLoadingView = nil;
fileLoaderManager = nil;

local gameObject = nil;

function Start(object)
    if jit and jit.off then jit.off() end
    math.randomseed(os.time());

    gameObject = object;
    gameSDK = GameSDK:new(gameObject, json);
    local version = gameSDK:getVersion();
    local versionPath = string.gsub(version, "%.", "_");
    print("#Game Start bigVersion versionPath",version,versionPath,"PlayerMode",LuaFramework.AppConst.PlayerMode);
    local filePath = UnityEngine.Application.persistentDataPath .. "/" .. versionPath .. "/";
    if(LuaFramework.AppConst.PlayerMode == 1)then
        LuaFramework.Util.AddResSearchPath(LuaFramework.AppConst.PcResPath);
    elseif(LuaFramework.AppConst.PlayerMode == 2)then
        LuaFramework.Util.AddResSearchPath(LuaFramework.AppConst.PcResPath);
    elseif(LuaFramework.AppConst.PlayerMode == 3)then
        -- 可写目录下的 (游戏包版本号目录，解压后文件) 换大版本号后 失效
        LuaFramework.Util.AddResSearchPath(filePath);
        -- 可写目录 (公共资源，例如 下载完未解压文件，扩展包解压后文件，升大版号后，不清除的文件)
        LuaFramework.Util.AddResSearchPath(UnityEngine.Application.persistentDataPath .. "/");
        -- APP目录(包升级，整个目录内容会清掉)
        LuaFramework.Util.AddResSearchPath(UnityEngine.Application.streamingAssetsPath .. "/");
    end
    -- fileLoaderManager = FileLoaderManager.New(UnityEngine.Application.persistentDataPath);
    fileLoaderManager = UnityEngine.GameObject.Find("Driver"):GetComponent(typeof(Driver)).fileLoaderManager;
    fileLoaderManager:SetUnCompressPath(filePath);

    -- 从上往下遍历
    LuaFramework.Util.Log("### 读取路径filePath:  "..filePath);
    LuaFramework.Util.Log("### 读取路径persistentDataPath:  "..UnityEngine.Application.persistentDataPath);
    LuaFramework.Util.Log("### 读取路径streamingAssetsPath:  "..UnityEngine.Application.streamingAssetsPath);
    LuaFramework.Util.Log("### 读取路径appDataPath:  "..UnityEngine.Application.dataPath);

    require "GameConfig"
    -- 提前require GameScene 防止下面删除操作
    require "GameScene"
    gameConfig = GameConfig:new();
    gameConfig.bigVersion = version;
    if gameConfig:isAndroid() == true or gameConfig:isIOS() == true then
        --设置sdkId
        gameConfig.sdkId = tonumber(gameSDK:getSdkId());
    end

    globalLoadingView = LoadingView:new(UnityEngine.GameObject.Find("loadingCanvas"),UnityEngine.GameObject.Find("loadingCamera"));
    globalLoadingView:show();
    globalLoadingView:updateLoading(0,"正在为您检查资源包更新");


    LuaFramework.Util.Log("### checkVerUrl   "..gameConfig:getCheckVerUrl());
    checkVer();
end

function checkVer()
    local checkVerUrl = gameConfig:getCheckVerUrl();
    -- LuaFramework.Util.Log("### checkVerUrl   "..checkVerUrl);

    StartCoroutine(function() loadVerUrl(checkVerUrl); end);
end

function loadVerUrl(url)
    local download = UnityEngine.WWW(url);
    Yield(download);
    local txt = download.text;
    if txt ~= nil and string.len(txt) > 0 then
        local data = json.decode(txt);
        if data ~= nil then
            globalLoadingView:updateLoading(10,"正在为您检查资源包更新");
            gameConfig.isShenhe = data.isShenhe == "1";
            gameConfig.configUrl = data.configUrl;
            if data.adminUrl ~= nil then
                gameConfig.errorUrl = data.adminUrl;
            end
            gameConfig.versionList = json.decode(data.versionList);
            if gameConfig.isShenhe == true then
                LuaFramework.Util.Log(tostring(os.time()));
                LuaFramework.Util.Log = function() end
            end
            LuaFramework.Util.Log("### checkVer:  "..txt);
            LuaFramework.Util.Log("### isShenhe: "..tostring(gameConfig.isShenhe).."   configUrl: "..tostring(gameConfig.configUrl).."   adminUrl: "..tostring(gameConfig.errorUrl));

            local delStr = data.delList;
            local delList = {};
            local vals = string.split(delStr, "\r\n");
            if vals ~= nil and #vals > 0 then
                for i=1, #vals do
                    local ps = string.split(vals[i], "|");
                    if ps ~= nil and #ps == 2 then
                        delList[ps[1]] = ps[2];
                    end
                end
            end
            gameConfig.delList = delList;

            local urlStr = data.urlList;
            local urlList = {};
            local vals = string.split(urlStr, "\r\n");
            if vals ~= nil and #vals > 0 then
                for i=1, #vals do
                    local ps = string.split(vals[i], "###");
                    if ps ~= nil and #ps == 2 then
                        urlList[ps[1]] = ps[2];
                    end
                end
            end
            gameConfig.urlList = urlList;

            checkDel();
        else
            LuaFramework.Util.Log("### checkVer load wrong");
            checkVer();
        end
    else
        LuaFramework.Util.Log("### checkVer load error");
        checkVer();
    end
end

function checkDel()
    --缓存代码清理
    globalLoadingView:updateLoading(20,"正在为您检查资源包更新");
    local str = gameSDK:readKV("codeVersion");
    local codeVersion = 1;
    if str ~= "" then codeVersion = tonumber(str);end
    LuaFramework.Util.Log("### codeVersion：  "..codeVersion);
    if codeVersion > 0 then
        local fileList;
        for k,v in pairs(gameConfig.delList) do
            if tonumber(k) == codeVersion then
                fileList = string.split(v, ",");
            end
        end
        if fileList ~= nil and #fileList > 0 then
            delFiles(fileList);
            gameSDK:writeKV("codeVersion", 0);
        end
    end
    -- 开始进入游戏流程
    enterGame();
end

function enterGame()
    if(isStart == true)then return; end
    globalLoadingView:updateLoading(30,"正在为您检查资源包更新");
    isStart = true;
    
    gameScene = GameScene:new(gameObject, gameConfig, gameSDK, json);
end


function callToUnity( data )
    gameSDK:callToUnity( data )
end

-- drive.cs 调用tick
function gameTick(delta)
    if gameScene ~= nil then
        gameScene:gameTick(delta);
    end
    if(globalLoadingView ~= nil)then
        globalLoadingView:tick();
    end
end

function delFiles(fileList)
    local versionPath = string.gsub(gameConfig.bigVersion, "%.", "_");
    for i=1, #fileList do
        if fileList[i] ~= "" then
            local versionFile = UnityEngine.Application.persistentDataPath .. "/" .. versionPath .. "/" .. fileList[i];
            if delFile(versionFile) == false then
                 versionFile = UnityEngine.Application.persistentDataPath .. "/" .. versionPath .. "/gamelua/" .. fileList[i];
                if delFile(versionFile) == false then
                    versionFile = UnityEngine.Application.persistentDataPath .. "/" .. fileList[i];
                    if delFile(versionFile) == false then
                        versionFile = UnityEngine.Application.persistentDataPath .. "/gamelua/" .. fileList[i];
                        delFile(versionFile);
                    end
                end
            end
        end
    end
end

function delFile(path)
    local isExists = System.IO.File.Exists(path);
    if isExists == true then
        LuaFramework.Util.Log("### delete file："..path);
        System.IO.File.Delete(path);
    end
    return isExists;
end


------------------------SDK-----------------------
GameSDK = class2("GameSDK");

function GameSDK:ctor(gameObj, json)
    self.gameObj = gameObj;
    self.json = json;
    self.sdk = self.gameObj:GetComponent(typeof(LuaFramework.SDKManager));
    self.handlers = {};
end

function GameSDK:ucall( code, data )
    return self.sdk:ucall( code, data );
end

function GameSDK:callToUnity( data )
    print("GameSDK:callToUnity", data);
    LuaFramework.Util.Log("#GameSDK callToUnity： "..data);
    jdata = self.json.decode(data);
    self:doCallback(jdata);
end

function GameSDK:addCallback( code, callBack, cbTarget )
    local list = self.handlers[code]
    if(list == nil) then
        list = {};
        self.handlers[code] = list;
    end
    local index = self:contain( list, callBack, cbTarget );
    if(index == -1) then
        local cbObj = { cb = callBack, tar = cbTarget };
        table.insert(list, cbObj);
    end
    return self;
end

function GameSDK:removeCallback( code, callBack, cbTarget )
    local list = self.handlers[code];
    if(list ~= nil) then
        local index = self:contain( list, callBack, cbTarget );
        if(index > -1) then
            table.remove(list, index)
        end
    end
    return self;
end

function GameSDK:contain( list, callBack, cbTarget )
    for i = 1, #list do
        local item = list[i];
        if(item.cb == callBack and item.tar == cbTarget) then
            return i;
        end
    end
    return -1;
end

function GameSDK:doCallback( jdata )
    if jdata ~= nil and jdata.code ~= nil then
        local list = self.handlers[tonumber(jdata.code)];
        if(list ~= nil) then
            for i = 1, #list do
                local item = list[i];
                if item ~= nil then
                    item.cb(item.tar, jdata.data);
                end
            end
        end
    end
end

function GameSDK:getVersion()
    local version = self:ucall(4);
    if version == nil or version == "" then
        version = "1.0";
    end
    return version;
end

function GameSDK:getSdkId()
    local sdkId = self:ucall(5);
    if sdkId == nil or sdkId == "" then
        sdkId = "0";
    end
    return sdkId;
end

function GameSDK:initSDK(callBack, callBackTarget)
    local initSDKCallback = function(target, data)
        self:removeCallback(2000, initSDKCallback, nil);
        if callBack ~= nil then
            callBack(callBackTarget, data);
        end
    end

    self:addCallback(2000, initSDKCallback);
    self:ucall(2000);
end

function GameSDK:getNetworkState()
    return self:ucall(6);
end

function GameSDK:readKV(key)
    if UnityEngine.Application.isMobilePlatform == true then
        return self:ucall(203,key);
    else
        return UnityEngine.PlayerPrefs.GetString(key);
    end
end

function GameSDK:writeKV(key,value)
    if UnityEngine.Application.isMobilePlatform == true then
        local data={key = key,value = value};
        local jdata = self.json.encode(data);
        self:ucall(202,jdata);
    else
        local v = tostring(value);
        UnityEngine.PlayerPrefs.SetString(key,v);
    end
end

function GameSDK:deleteKey(key)
    if UnityEngine.Application.isMobilePlatform == true then
        self:ucall(204,key);
    else
        UnityEngine.PlayerPrefs.DeleteKey(key);
    end
end

function GameSDK:neverSleep()
    self:ucall(104);
end

function GameSDK:openSleep()
    self:ucall(105);
end

function GameSDK:restartGame()
    self:ucall(114);
end


------------------------加载界面----------------------

LoadingView = class2("LoadingView");

function LoadingView:ctor(go,camGo)
    self.gameObject = go;
    self.transform = go.transform;
    self.camGo = camGo;
    self.camTr = camGo.transform;

    self._barLabel = nil;
    self._barTran = nil;
    self._barHeight = 0;
    self._tipBoxGo = nil;
    self._descriptLabel = nil;
    self._callBack = nil;
    self._cbTarget = nil;
    self._isInMotion = false;
    self._defaultStr = "正在启动游戏...%d%%";
    self._specailStr = nil;

    self._currentValue = 0;
    self._targetValue = 0;
    self._delValue = 0;
    self._bgImg = nil;
    self._vector2 = Vector2.New();
    self._transSizeX = 720;

    local size = self.transform.sizeDelta;
    self._transSizeX = size.x;
    local loadingViewTrans = self.transform:Find("loadingView");
    self._bgImg = loadingViewTrans:Find("bgImg"):GetComponent(typeof(UnityEngine.UI.Image));
    self._barLabel = loadingViewTrans:Find("barLabel"):GetComponent(typeof(UnityEngine.UI.Text));
    self._barLabel.text = "正在启动游戏...";
    local bar = loadingViewTrans:Find("bar"):GetComponent(typeof(UnityEngine.UI.Image));
    self._barTran = bar.transform;
    self._barHeight = self._barTran.sizeDelta.y;
    self._vector2:Set(1,self._barHeight);
    self._barTran.sizeDelta = self._vector2;

    local tipBoxTrans = loadingViewTrans:Find("tipBox");
    self._tipBoxGo = tipBoxTrans.gameObject;
    local sureBtn = tipBoxTrans:Find("sureBtn"):GetComponent(typeof(TouchTrigger));
    sureBtn:setTouchEnabled(true);
    sureBtn:setLuaCallback(self.onSureBtnClick,self);
    local cancelBtn = tipBoxTrans:Find("cancelBtn"):GetComponent(typeof(TouchTrigger));
    cancelBtn:setTouchEnabled(true);
    cancelBtn:setLuaCallback(self.onCancelBtnClick,self);
    self._descriptLabel = tipBoxTrans:Find("desLabel"):GetComponent(typeof(UnityEngine.UI.Text));
    -- 添加事件，为了下载资源弹窗点击使用，现不弹窗，废弃（而且restart会一直叠加事件)
    -- self:addEventSystem();
end

-- function LoadingView:addEventSystem()
--     local go = UnityEngine.GameObject("EventSystem");
--     go.transform:SetParent(self.transform);
--     local eventSystem = go:AddComponent(typeof(UnityEngine.EventSystems.EventSystem));
--     eventSystem.sendNavigationEvents = false;
--     eventSystem.pixelDragThreshold = 15;
--     go:AddComponent(typeof(UnityEngine.EventSystems.StandaloneInputModule));
-- end

function LoadingView:updateLoading(progress,tipStr)
    if gameConfig:isIOSShenhe() then
        self._specailStr = "进入中";
    else
        self._specailStr = tipStr;
    end
    self:updateProgress(progress);
end

function LoadingView:showAlert(message,callBack,cbTarget)
    self._tipBoxGo:SetActive(true);
    self._descriptLabel.text = message;
    self._callBack = callBack;
    self._cbTarget = cbTarget;
end

function LoadingView:onSureBtnClick(self)
    self._tipBoxGo:SetActive(false);
    if(self._callBack ~= nil)then
        self._callBack(self._cbTarget,1);
    end
end

function LoadingView:onCancelBtnClick(self)
    self._tipBoxGo:SetActive(false);
    if(self._callBack ~= nil)then
        self._callBack(self._cbTarget,0);
    end
end

function LoadingView:show()
    self.gameObject:SetActive(true);
    self.camGo:SetActive(true);
    if(self._specailStr ~= nil)then
        self._barLabel.text = string.format(self._specailStr.."...%d%%",self._currentValue);
    else
        self._barLabel.text = string.format(self._defaultStr,self._currentValue);
    end
end

function LoadingView:updateProgress(progress)
    self._targetValue = progress;
    self._delValue = (self._targetValue - self._currentValue) / 15;
    self:start();
end

function LoadingView:start()
    self._isInMotion = true;
    -- self:tick();
end

function LoadingView:stop()
    self._isInMotion = false;
end

function LoadingView:tick()
    if(self._isInMotion == false)then
        return;
    end
    self._currentValue = self._currentValue + self._delValue;
    if(self._currentValue >= self._targetValue)then
        self._currentValue = self._targetValue;
        self:stop();
    end
    if(self._specailStr ~= nil)then
        self._barLabel.text = string.format(self._specailStr.."...%d%%",self._currentValue);
        -- self._barLabel.text = string.format(self._specailStr,self._currentValue);
    else
        self._barLabel.text = string.format(self._defaultStr,self._currentValue);
    end

    local w = self._currentValue / 100 * self._transSizeX;
    print("进度条宽度 == ",w,self._currentValue,self._targetValue,self._transSizeX,self._specailStr);
    self._vector2:Set(w,self._barHeight);
    self._barTran.sizeDelta = self._vector2;
end

function LoadingView:hide()
    self:stop();
    self._barLabel.text = string.format(self._defaultStr,1);
    self._vector2:Set(1,self._barHeight);
    self._barTran.sizeDelta = self._vector2;
    self._targetValue = 0;
    self._currentValue = 0;
    self.gameObject:SetActive(false);
    self.camGo:SetActive(false);
end